#version 330 core

#define TEXTURE_LOD_BIAS 0.0

////////////////////////////////////////////////////////////////////////////////////
////////// POSITION ATTRIBUTE - FRAGMENT
////////////////////////////////////////////////////////////////////////////////////
#define nop() { }

in vec4 v_position;
#define pullPosition() { return v_position; }


in vec2 v_texCoord0;

// Uniforms which are always available
uniform mat4 u_projViewTrans;

uniform mat4 u_worldTrans;

uniform vec4 u_cameraPosition;

uniform mat3 u_normalMatrix;

// Varyings computed in the vertex shader
in float v_opacity;
in float v_alphaTest;

#ifdef diffuseTextureFlag
uniform sampler2D u_diffuseTexture;
#endif

#ifdef normalTextureFlag
uniform sampler2D u_normalTexture;
#endif


// AMBIENT LIGHT
in vec3 v_ambientLight;

// CLOUD TEXTURE
#if defined(diffuseTextureFlag) && defined(normalTextureFlag)
// We have clouds and transparency
vec4 fetchCloudColor(vec2 texCoord, vec4 defaultValue) {
    vec4 cloud = texture(u_diffuseTexture, texCoord, TEXTURE_LOD_BIAS);
    vec4 trans = texture(u_normalTexture, texCoord, TEXTURE_LOD_BIAS);
    return vec4(cloud.rgb, 1.0 - (trans.r + trans.g + trans.b) / 3.0);
}
#elif defined(diffuseTextureFlag)
// Only clouds, we use value as transp
vec4 fetchCloudColor(vec2 texCoord, vec4 defaultValue) {
    vec4 cloud = texture(u_diffuseTexture, texCoord, TEXTURE_LOD_BIAS);
    // Smooth towards the poles
    float smoothing = smoothstep(0.01, 0.07, texCoord.y);
    return vec4(2.0 * cloud.rgb, smoothing * (cloud.r + cloud.g + cloud.b) / 3.0);
}
    #else
vec4 fetchCloudColor(vec2 texCoord, vec4 defaultValue) {
    return defaultValue;
}
    #endif// diffuseTextureFlag && diffuseColorFlag

in vec3 v_lightDir;
in vec3 v_lightCol;
in vec3 v_viewDir;

in float v_depth;
out vec4 fragColor;

void main() {
    vec2 g_texCoord0 = v_texCoord0;

    vec4 cloud = fetchCloudColor(g_texCoord0, vec4(0.0, 0.0, 0.0, 0.0));
    vec3 ambient = v_ambientLight;
    float ambient_val = (ambient.r + ambient.g + ambient.b) / 3.0;

    // Normal in pixel space
    vec3 N = vec3(0.0, 0.0, 1.0);
    vec3 L = normalize(v_lightDir);
    float NL = max(0.0, dot(N, L));

    vec3 cloudColor = (v_lightCol * cloud.rgb);
    fragColor = vec4(cloudColor, cloud.a *  v_opacity * clamp(NL + ambient_val, 0.0, 1.0));

    fragColor = clamp(fragColor, 0.0, 1.0);

    gl_FragDepth = v_depth;
}